/*******************************************************************************
*
* E M B E D D E D   W I Z A R D   P R O J E C T
*
*                                                Copyright (c) TARA Systems GmbH
*                                    written by Paul Banach and Manfred Schweyer
*
********************************************************************************
*
* This software is delivered "as is" and shows the usage of other software
* components. It is provided as an example software which is intended to be
* modified and extended according to particular requirements.
*
* TARA Systems hereby disclaims all warranties and conditions with regard to the
* software, including all implied warranties and conditions of merchantability
* and non-infringement of any third party IPR or other rights which may result
* from the use or the inability to use the software.
*
********************************************************************************
*
* DESCRIPTION:
*   This file is part of the interface (glue layer) between an Embedded Wizard
*   generated UI application and the board support package (BSP) of a dedicated
*   target.
*   This template is responsible to establish a console connection in order
*   to send debug messages to a PC terminal tool, or to receive key events
*   for the UI application.
*
*******************************************************************************/

#include <stdio.h>
#include <stdint.h>
#include <stdbool.h>

#include "ewconfig.h"
#include "ew_bsp_console.h"

#include "r_sci_rx_pinset.h"
#include "r_sci_rx_if.h"

static sci_hdl_t Console;
static volatile uint32_t  ClearToSend = 1;

static void SciCallback(void* aArgs)
{
  sci_cb_args_t* args;
  args = (sci_cb_args_t*)aArgs;

  if (args->event == SCI_EVT_TEI)
  {
    ClearToSend = 1;
  }
}


/*******************************************************************************
* FUNCTION:
*   EwBspConsoleInit
*
* DESCRIPTION:
*   The function EwBspConsoleInit initializes a console connection via UART/USART
*   interface used to print error and trace messages from an Embedded Wizard
*   GUI application.
*   This implementation uses the following configuration:
*   - BaudRate = 115200 baud
*   - Word Length = 8 Bits
*   - One Stop Bit
*   - No parity
*   - Hardware flow control disabled (RTS and CTS signals)
*   - Receive and transmit enabled
*
* ARGUMENTS:
*   None
*
* RETURN VALUE:
*   None
*
*******************************************************************************/
void EwBspConsoleInit( void )
{
  sci_cfg_t config;

  /* initialize the I/O port pins for communication on this SCI channel */
  R_SCI_PinSet_SCI2();

  /* configuration of SCI async mode (UART) */
  config.async.baud_rate = 115200;
  config.async.clk_src = SCI_CLK_INT;
  config.async.data_size = SCI_DATA_8BIT;
  config.async.parity_en = SCI_PARITY_OFF;
  config.async.parity_type = SCI_EVEN_PARITY;
  config.async.stop_bits = SCI_STOPBITS_1;
  config.async.int_priority = 2;

  /* open SCI channel */
  ClearToSend = 1;
  R_SCI_Open( SCI_CH2, SCI_MODE_ASYNC, &config, &SciCallback, &Console );
}


/*******************************************************************************
* FUNCTION:
*   EwBspConsoleDone
*
* DESCRIPTION:
*   The function EwBspConsoleDone terminates a console connection.
*
* ARGUMENTS:
*   None
*
* RETURN VALUE:
*   None
*
*******************************************************************************/
void EwBspConsoleDone( void )
{
  R_SCI_Close( Console );
}


/*******************************************************************************
* FUNCTION:
*   EwBspConsolePutCharacter
*
* DESCRIPTION:
*   The function EwBspConsolePutCharacter sends the given character to the
*   console interface.
*
* ARGUMENTS:
*   aCharacter - The character to be send via the console interface.
*
* RETURN VALUE:
*   None
*
*******************************************************************************/
void EwBspConsolePutCharacter( unsigned char aCharacter )
{
  /* unfortunately, FIT SCI driver cannot receive next character while data
     transfer is ongoing */
  while ( ClearToSend == 0 )
    ;

  /* transfer next byte */
  ClearToSend = 0;
  R_SCI_Send( Console, &aCharacter, 1 );
}


/*******************************************************************************
* FUNCTION:
*   EwBspConsoleGetCharacter
*
* DESCRIPTION:
*   The function EwBspConsoleGetCharacter returns the current character from the
*   console interface. If no character is available within the input buffer, 0
*   is returned.
*
* ARGUMENTS:
*   None
*
* RETURN VALUE:
*   Current character from console input buffer or 0.
*
*******************************************************************************/
unsigned char EwBspConsoleGetCharacter( void )
{
  uint8_t ch;

  /* try to read one byte */
  if ( R_SCI_Receive( Console, &ch, 1 ) == SCI_SUCCESS )
    return ch;
  return 0;
}


/* msy */
